@@ -1,30 +1,5 @@
-/* libmpdclient
-   (c) 2003-2018 The Music Player Daemon Project
-   This project's homepage is: http://www.musicpd.org
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright
-   notice, this list of conditions and the following disclaimer.
-
-   - Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// SPDX-License-Identifier: BSD-2-Clause
+// Copyright The Music Player Daemon Project
 
 /*! \file
  * \brief Database
@@ -51,6 +26,8 @@ extern "C" {
  * MPD.  They are returned without metadata.  This is a rather
  * expensive operation, because the response may be large.
  *
+ * To read the response, you may use mpd_recv_entity().
+ *
  * @param connection the connection to MPD
  * @param path an optional base path for the query
  * @return true on success, false on error
@@ -80,14 +57,14 @@ mpd_send_list_all_meta(struct mpd_connection *connection, const char *path);
  * To read the response, you may use mpd_recv_entity().
  *
  * @param connection the connection to MPD
- * @param path the directory to be listed
+ * @param path an optional directory to be listed
  * @return true on success, false on error
  */
 bool
 mpd_send_list_meta(struct mpd_connection *connection, const char *path);
 
 /**
- * Lists the contents of the specified directory, including files are
+ * Lists the contents of the specified directory, including files that are
  * not recognized by MPD (command "listfiles").
  *
  * To read the response, you may use mpd_recv_entity().  All regular
@@ -141,7 +118,7 @@ bool
 mpd_send_rescan(struct mpd_connection *connection, const char *path);
 
 /**
- * Receives the id the of the update job which was submitted by
+ * Receives the id of the update job which was submitted by
  * mpd_send_update().
  *
  * @param connection the connection to MPD
