@@ -1,34 +1,5 @@
-/* libmpdclient
-   (c) 2003-2010 The Music Player Daemon Project
-   This project's homepage is: http://www.musicpd.org
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright
-   notice, this list of conditions and the following disclaimer.
-
-   - Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-
-   - Neither the name of the Music Player Daemon nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// SPDX-License-Identifier: BSD-3-Clause
+// Copyright The Music Player Daemon Project
 
 /*! \file
  * \brief MPD client library
@@ -38,14 +9,16 @@
  * Do not include this header directly.  Use mpd/client.h instead.
  */
 
-#ifndef MPD_DB_H
-#define MPD_DB_H
+#ifndef LIBMPDCLIENT_SEARCH_H
+#define LIBMPDCLIENT_SEARCH_H
 
-#include <mpd/connection.h>
-#include <mpd/tag.h>
-#include <mpd/compiler.h>
+#include "connection.h"
+#include "tag.h"
+#include "position.h"
+#include "compiler.h"
 
 #include <stdbool.h>
+#include <time.h>
 
 /**
  * This type is not yet used, it is reserved for a future protocol
@@ -90,6 +63,22 @@ mpd_search_db_songs(struct mpd_connection *connection, bool exact);
 bool
 mpd_search_add_db_songs(struct mpd_connection *connection, bool exact);
 
+/**
+ * Search for songs in the database and adds the result to a playlist.
+ * Constraints may be specified with mpd_search_add_tag_constraint().
+ * Send the search command with mpd_search_commit().
+ *
+ * @param connection the connection to MPD
+ * @param playlist_name the name of the playlist where songs shall be
+ * added
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.17.
+ */
+bool
+mpd_search_add_db_songs_to_playlist(struct mpd_connection *connection,
+				    const char *playlist_name);
+
 /**
  * Search for songs in the queue.
  * Constraints may be specified with mpd_search_add_tag_constraint().
@@ -127,6 +116,34 @@ mpd_search_db_tags(struct mpd_connection *connection, enum mpd_tag_type type);
  */
 bool mpd_count_db_songs(struct mpd_connection *connection);
 
+/**
+ * Gathers statistics on a set of songs in the database.
+ * This is the case insensitive variant of mpd_count_db_songs().
+ * Constraints may be specified with mpd_search_add_tag_constraint().
+ * Send the command with mpd_search_commit(), and read the response
+ * with mpd_recv_stats().
+ *
+ * @param connection the connection to MPD
+ * @return true on success, false on error
+ */
+bool
+mpd_searchcount_db_songs(struct mpd_connection *connection);
+
+/**
+ * Limit the search to a certain directory.
+ *
+ * @param connection a #mpd_connection
+ * @param oper reserved, pass #MPD_OPERATOR_DEFAULT
+ * @param value the URI relative to the music directory
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.9
+ */
+bool
+mpd_search_add_base_constraint(struct mpd_connection *connection,
+			       enum mpd_operator oper,
+			       const char *value);
+
 /**
  * Add a constraint on the song's URI.
  *
@@ -168,6 +185,120 @@ mpd_search_add_any_tag_constraint(struct mpd_connection *connection,
 				  enum mpd_operator oper,
 				  const char *value);
 
+/**
+ * Limit the search to files modified after the given time stamp.
+ *
+ * @param connection a #mpd_connection
+ * @param oper reserved, pass #MPD_OPERATOR_DEFAULT
+ * @param value the reference time stamp
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.10
+ */
+bool
+mpd_search_add_modified_since_constraint(struct mpd_connection *connection,
+					 enum mpd_operator oper,
+					 time_t value);
+
+/**
+ * Limit the search to files added after the given time stamp.
+ *
+ * @param connection a #mpd_connection
+ * @param oper reserved, pass #MPD_OPERATOR_DEFAULT
+ * @param value the reference time stamp
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.21, MPD 0.24
+ */
+bool
+mpd_search_add_added_since_constraint(struct mpd_connection *connection,
+				      enum mpd_operator oper,
+				      time_t value);
+
+/**
+ * Add an expression string.
+ *
+ * @param connection a #mpd_connection
+ * @param expression the expression string; must be enclosed in
+ * parentheses
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.15, MPD 0.21
+ */
+bool
+mpd_search_add_expression(struct mpd_connection *connection,
+			  const char *expression);
+
+/**
+ * Group the results by the specified tag.
+ *
+ * @param connection a #mpd_connection
+ * @param type the tag type to group by
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.12, MPD 0.19
+ */
+bool
+mpd_search_add_group_tag(struct mpd_connection *connection,
+			 enum mpd_tag_type type);
+
+/**
+ * Sort the results by the specified named attribute.
+ *
+ * @param connection a #mpd_connection
+ * @param name the attribute name to sort with; can be a tag name or
+ * "Last-Modified"
+ * @param descending sort in reverse order?
+ * @return true on success, false on error
+ *
+ * @since MPD 0.21, libmpdclient 2.15
+ */
+bool
+mpd_search_add_sort_name(struct mpd_connection *connection,
+			 const char *name, bool descending);
+
+/**
+ * Sort the results by the specified tag.
+ *
+ * @param connection a #mpd_connection
+ * @param type the tag type to sort with
+ * @param descending sort in reverse order?
+ * @return true on success, false on error
+ *
+ * @since MPD 0.21, libmpdclient 2.11; #descending since libmpdclient
+ * 2.15
+ */
+bool
+mpd_search_add_sort_tag(struct mpd_connection *connection,
+			enum mpd_tag_type type, bool descending);
+
+/**
+ * Request only a portion of the result set.
+ *
+ * @param connection a #mpd_connection
+ * @param start the start offset (including)
+ * @param end the end offset (not including)
+ * @return true on success, false on error
+ *
+ * @since libmpdclient 2.10
+ */
+bool
+mpd_search_add_window(struct mpd_connection *connection,
+		      unsigned start, unsigned end);
+
+/**
+ * Adds the search to the specified position in the queue.
+ *
+ * @param connection a #mpd_connection
+ * @param position the position in the queue
+ * @param whence how to interpret the position parameter
+ * 
+ * @since libmpdclient 2.20
+ */
+bool
+mpd_search_add_position(struct mpd_connection *connection,
+			unsigned position, enum mpd_position_whence whence);
+
 /**
  * Starts the real search with constraints added with
  * mpd_search_add_constraint().
