@@ -1,34 +1,5 @@
-/* libmpdclient
-   (c) 2003-2010 The Music Player Daemon Project
-   This project's homepage is: http://www.musicpd.org
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright
-   notice, this list of conditions and the following disclaimer.
-
-   - Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-
-   - Neither the name of the Music Player Daemon nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// SPDX-License-Identifier: BSD-3-Clause
+// Copyright The Music Player Daemon Project
 
 /*! \file
  * \brief MPD client library
@@ -39,7 +10,7 @@
 #ifndef MPD_STATUS_H
 #define MPD_STATUS_H
 
-#include <mpd/compiler.h>
+#include "compiler.h"
 
 #include <stdbool.h>
 
@@ -60,6 +31,56 @@ enum mpd_state {
 	MPD_STATE_PAUSE = 3,
 };
 
+/**
+ * MPD's single state.
+ *
+ * @since libmpdclient 2.18, MPD 0.21.
+ */
+enum mpd_single_state {
+	/** disabled */
+	MPD_SINGLE_OFF = 0,
+
+	/** enabled */
+	MPD_SINGLE_ON,
+
+	/**
+	 * enables single state (#MPD_SINGLE_ONESHOT) for a single song, then
+	 * MPD disables single state (#MPD_SINGLE_OFF) if the current song
+	 * has played and there is another song in the current playlist
+	 *
+	 * @since MPD 0.21
+	 **/
+	MPD_SINGLE_ONESHOT,
+
+	/** Unknown state */
+	MPD_SINGLE_UNKNOWN,
+};
+
+/**
+ * MPD's consume state.
+ *
+ * @since libmpdclient 2.21, MPD 0.24.
+ */
+enum mpd_consume_state {
+	/** disabled */
+	MPD_CONSUME_OFF = 0,
+
+	/** enabled */
+	MPD_CONSUME_ON,
+
+	/**
+	 * enables consume state (#MPD_CONSUME_ONESHOT) for a single song, then
+	 * MPD disables consume state (#MPD_CONSUME_OFF) if the current song
+	 * has played.
+	 *
+	 * @since MPD 0.24
+	 **/
+	MPD_CONSUME_ONESHOT,
+
+	/** Unknown state */
+	MPD_CONSUME_UNKNOWN,
+};
+
 struct mpd_connection;
 struct mpd_pair;
 struct mpd_audio_format;
@@ -148,14 +169,89 @@ bool
 mpd_status_get_random(const struct mpd_status *status);
 
 /**
- * Returns true if single mode is on.
+ * Returns the current state of single mode on MPD.
+ *
+ * If the state is #MPD_SINGLE_ONESHOT, MPD will transition to #MPD_SINGLE_OFF
+ * after a song is played and if there is another song in the queue. The
+ * #mpd_status object will not be updated accordingly. In this case, you need
+ * to call mpd_send_status() and mpd_recv_status() again.
+ *
+ * @since MPD 0.21, libmpdclient 2.18.
+ */
+mpd_pure
+enum mpd_single_state
+mpd_status_get_single_state(const struct mpd_status *status);
+
+/**
+ * Looks up the name of the specified single mode.
+ *
+ * @return the name, or NULL if the single mode is not valid
+ *
+ * @since libmpdclient 2.21.
+ */
+const char *
+mpd_lookup_single_state(enum mpd_single_state state);
+
+/**
+ * Parse the string to check which single mode it contains.
+ *
+ * @return the single mode enum
+ *
+ * @since libmpdclient 2.21.
+ */
+enum mpd_single_state
+mpd_parse_single_state(const char *p);
+
+/**
+ * This function is deprecated as it does not distinguish the states of
+ * the single mode (added to MPD 0.21). Call mpd_status_get_single_state() in
+ * its place.
+ *
+ * Returns true if single mode is either on or in oneshot.
  */
 mpd_pure
 bool
 mpd_status_get_single(const struct mpd_status *status);
 
 /**
- * Returns true if consume mode is on.
+ * Returns the current state of consume mode on MPD.
+ *
+ * If the state is #MPD_CONSUME_ONESHOT, MPD will transition to #MPD_CONSUME_OFF
+ * after a song is played. The #mpd_status object will not be updated accordingly.
+ * In this case, you need to call mpd_send_status() and mpd_recv_status() again.
+ *
+ * @since MPD 0.24, libmpdclient 2.21.
+ */
+mpd_pure
+enum mpd_consume_state
+mpd_status_get_consume_state(const struct mpd_status *status);
+
+/**
+ * Looks up the name of the specified consume mode.
+ *
+ * @return the name, or NULL if the consume mode is not valid
+ *
+ * @since libmpdclient 2.21.
+ */
+const char *
+mpd_lookup_consume_state(enum mpd_consume_state state);
+
+/**
+ * Parse the string to check which consume mode it contains.
+ *
+ * @return the consume mode enum
+ *
+ * @since libmpdclient 2.21.
+ */
+enum mpd_consume_state
+mpd_parse_consume_state(const char *p);
+
+/**
+ * This function is deprecated as it does not distinguish the states of
+ * the consume mode (added to MPD 0.24). Call mpd_status_get_consume_state() in
+ * its place.
+ *
+ * Returns true if consume mode is either on or in oneshot.
  */
 mpd_pure
 bool
@@ -193,15 +289,19 @@ unsigned
 mpd_status_get_crossfade(const struct mpd_status *status);
 
 /**
- * Returns mixrampdb setting in db.
+ * Returns mixrampdb setting in db. 0 means mixrampdb is disabled.
+ *
+ * @since libmpdclient 2.2
  */
 mpd_pure
 float
 mpd_status_get_mixrampdb(const struct mpd_status *status);
 
 /**
- * Returns mixrampdelay setting in seconds.  Negative means mixramp is
+ * Returns mixrampdelay setting in seconds.  Negative means mixrampdelay is
  * disabled.
+ *
+ * @since libmpdclient 2.2
  */
 mpd_pure
 float
@@ -210,22 +310,48 @@ mpd_status_get_mixrampdelay(const struct mpd_status *status);
 /**
  * Returns the position of the currently playing song in the queue
  * (beginning with 0) if a song is currently selected (always the case when
- * state is PLAY or PAUSE).  If there is no current song, -1 is returned.
+ * state is MPD_STATE_PLAY or MPD_STATE_PAUSE).  If there is no current song,
+ * -1 is returned.
  */
 mpd_pure
 int
 mpd_status_get_song_pos(const struct mpd_status *status);
 
 /**
- * Returns the id of the currently song.  If there is no current song,
+ * Returns the id of the current song.  If there is no current song,
  * -1 is returned.
  */
+mpd_pure
 int
 mpd_status_get_song_id(const struct mpd_status *status);
 
 /**
+ * The same as mpd_status_get_song_pos(), but for the next song to be
+ * played.
+ *
+ * @since libmpdclient 2.7
+ */
+mpd_pure
+int
+mpd_status_get_next_song_pos(const struct mpd_status *status);
+
+/**
+ * Returns the id of the next song to be played.  If it is not known, -1 is
+ * returned.
+ *
+ * @since libmpdclient 2.7
+ */
+mpd_pure
+int
+mpd_status_get_next_song_id(const struct mpd_status *status);
+
+/**
+ * This function uses a deprecated feature of MPD, call
+ * mpd_status_get_elapsed_ms() instead.
+ *
  * Returns time in seconds that have elapsed in the currently playing/paused
- * song
+ * song.
+ *
  */
 mpd_pure
 unsigned
@@ -234,6 +360,8 @@ mpd_status_get_elapsed_time(const struct mpd_status *status);
 /**
  * Returns time in milliseconds that have elapsed in the currently
  * playing/paused song.
+ *
+ * @since libmpdclient 2.1
  */
 mpd_pure
 unsigned
@@ -268,6 +396,14 @@ mpd_pure
 unsigned
 mpd_status_get_update_id(const struct mpd_status *status);
 
+/**
+ * Returns the name of the current partition or NULL if the server did
+ * not send a name.
+ */
+mpd_pure
+const char *
+mpd_status_get_partition(const struct mpd_status *status);
+
 /**
  * Returns the error message
  */
